"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFileLineByHashSum = exports.isAnalyzerErrorCode = exports.getHashSumFromString = exports.getStringWithoutDiagnosticComment = void 0;
/**
 * Убирает из строки комментарий вида "//-V505"
 * @param str Строка файла
 * @returns
 */
function getStringWithoutDiagnosticComment(str) {
    let trimmedString = str.trim();
    const indexOfComment = trimmedString.lastIndexOf("//-V");
    if (indexOfComment !== -1) {
        trimmedString = str.slice(0, indexOfComment);
    }
    return trimmedString.trim();
}
exports.getStringWithoutDiagnosticComment = getStringWithoutDiagnosticComment;
/**
 * Возвращает хэш сумму строки по алгоритму, взятому из PVS-Studio_Cmd
 * @param str
 * @returns
 */
function getHashSumFromString(str) {
    let continueSearch = true;
    do {
        continueSearch = false;
        let index = str.lastIndexOf("//-");
        if (index !== -1) {
            let subStr = str.substring(index + 3);
            subStr = subStr.trimEnd();
            continueSearch = isAnalyzerErrorCode(subStr);
            if (continueSearch) {
                str = str.replaceAll(subStr, "");
            }
            else {
                str = str.replaceAll("//-", "");
            }
        }
    } while (continueSearch);
    let sum = 0;
    for (let char of str) {
        const charCode = char.charCodeAt(0);
        if (charCode >= 0 && char !== " " && char !== "\t") {
            const hiBit = (sum & 0x80000000) !== 0;
            sum <<= 1;
            sum = sum & 0xffffffff; // костыль, чтоб не переполнить 32 бита
            sum ^= charCode;
            if (hiBit) {
                sum ^= 0x00000001;
            }
        }
    }
    return sum;
}
exports.getHashSumFromString = getHashSumFromString;
function isAnalyzerErrorCode(errStr) {
    if (errStr.length !== 4 && errStr.length !== 5) {
        return false;
    }
    if (errStr[0] !== 'V' && errStr[0] !== 'v') {
        return false;
    }
    for (var i = 1; i <= 3; i++) {
        if (isNaN(Number(errStr[i]))) {
            return false;
        }
    }
    if (errStr.length === 5) {
        if (isNaN(Number(errStr[4]))) {
            return false;
        }
    }
    return true;
}
exports.isAnalyzerErrorCode = isAnalyzerErrorCode;
/**
   * Выдает строку, на которой надо открыть файл, с учетом хэш суммы строки
   * @param editor Открытый файл
   * @param file Объект, где есть хэш строк
   * @returns
   */
function getFileLineByHashSum(editor, file) {
    return __awaiter(this, void 0, void 0, function* () {
        // @ts-ignore
        const lineHashReportCurrent = file.navigation.currentLine;
        // @ts-ignore
        const lineHashReportPrev = file.navigation.previousLine;
        // @ts-ignore
        const lineHashReportNext = file.navigation.nextLine;
        const allText = editor.document.getText();
        const allLinesCount = allText.split(/\r?\n/gm).length;
        const linesFrom = file.line - 10 > 0 ? file.line - 10 : 0 + 1;
        const linesTo = file.line + 10 < allLinesCount ? file.line + 10 : allLinesCount - 2;
        const lineCur = file.line - 1 > 0 ? file.line - 1 : linesFrom;
        const linePrev = file.line - 2 > 0 ? file.line - 2 : linesFrom;
        const lineNext = file.line > 0 ? file.line : linesFrom;
        const lineString = editor.document.lineAt(lineCur).text;
        const lineStringPrev = editor.document.lineAt(linePrev).text;
        const lineStringNext = editor.document.lineAt(lineNext).text;
        const lineHash = getHashSumFromString(lineString);
        const lineHashPrev = getHashSumFromString(lineStringPrev);
        const lineHashNext = getHashSumFromString(lineStringNext);
        if (lineHash === lineHashReportCurrent &&
            lineHashPrev === lineHashReportPrev &&
            lineHashNext === lineHashReportNext) {
            return file.line;
        }
        else {
            let lineIsFound = false;
            // Ищем нужную строку +-10 строк от изначальной
            for (let i = linesFrom; i <= linesTo; i++) {
                const string = editor.document.lineAt(i).text;
                const prevString = editor.document.lineAt(i - 1).text;
                const nextString = editor.document.lineAt(i + 1).text;
                let _lineHash = 0;
                let _lineHashPrev = 0;
                let _lineHashNext = 0;
                if (string === undefined) {
                    continue;
                }
                _lineHash = getHashSumFromString(string);
                _lineHashPrev = getHashSumFromString(prevString);
                _lineHashNext = getHashSumFromString(nextString);
                if (_lineHash === lineHashReportCurrent &&
                    _lineHashPrev === lineHashReportPrev &&
                    _lineHashNext === lineHashReportNext) {
                    return i + 1;
                }
            }
            if (!lineIsFound) {
                return file.line;
            }
        }
    });
}
exports.getFileLineByHashSum = getFileLineByHashSum;
//# sourceMappingURL=hash.js.map