"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getJsonPathByUser = exports.excludePath = void 0;
const pathNode = require("path");
const fs = require("fs");
const os = require("os");
const vscode = require("vscode");
const PluginSettingsManager_1 = require("../PluginSettingsManager");
function excludePath(path) {
    return __awaiter(this, void 0, void 0, function* () {
        const settings = PluginSettingsManager_1.PluginSettingsManager.settings;
        let excludedPaths = (settings === null || settings === void 0 ? void 0 : settings.excludedPaths) || [];
        let isDirectory;
        try {
            const stats = yield fs.promises.stat(path);
            isDirectory = stats.isDirectory();
        }
        catch (_a) {
            pathNode.basename(path).includes(".") ? isDirectory = false : isDirectory = true;
        }
        let pathToExclude = isDirectory ? pathNode.join(path, "*") : path;
        excludedPaths = [...new Set([...excludedPaths, pathToExclude])];
        PluginSettingsManager_1.PluginSettingsManager.saveSettings(Object.assign(Object.assign({}, settings), { excludedPaths }));
    });
}
exports.excludePath = excludePath;
function getJsonPathByUser() {
    return __awaiter(this, void 0, void 0, function* () {
        // Выбираем файл через окно выбора файлов
        const fileObj = yield vscode.window.showOpenDialog({ filters: { json: ["json"] } });
        // Если файл не выбран
        if (!fileObj) {
            return;
        }
        // Получаем путь до файла
        let { path } = fileObj[0];
        if ((path.startsWith("\\") || path.startsWith("/")) && os.platform() === "win32") {
            path = path.slice(1);
        }
        // Проверяем, .json ли это
        if (path.split(".")[path.split(".").length - 1] !== "json") {
            vscode.window.showErrorMessage("Incorrect format of PVS-Studio report.");
            return;
        }
        return path;
    });
}
exports.getJsonPathByUser = getJsonPathByUser;
//# sourceMappingURL=pathUtils.js.map